"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CL4RScheme = void 0;
var BN = require("bn.js");
var decimal_js_1 = require("decimal.js");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var utils_1 = require("../utils");
var operation_1 = require("../operation");
// same address for all networks (mainnet,kovan,xdai,rinkeby)
var CT4RRedeemerAddress = '0x829BDfd41d517f57E5eBf13AD0E181351cb16a96';
var CT4RRedeemerABI = [
    {
        constant: false,
        inputs: [
            {
                internalType: 'contract ContinuousLocking4Reputation',
                name: 'clt4Reputation',
                type: 'address'
            },
            {
                components: [
                    {
                        internalType: 'address',
                        name: 'beneficiary',
                        type: 'address'
                    },
                    {
                        internalType: 'uint256',
                        name: 'id',
                        type: 'uint256'
                    }
                ],
                internalType: 'struct NectarReputationRedeemer.Redeem[]',
                name: 'clt4rRedeems',
                type: 'tuple[]'
            }
        ],
        name: 'redeemContinuousLocking4Reputation',
        outputs: [],
        payable: false,
        stateMutability: 'nonpayable',
        type: 'function'
    }
];
var CL4RScheme = /** @class */ (function () {
    function CL4RScheme(scheme) {
        this.scheme = scheme;
    }
    /**
     * getRepuationRewardForBatch
     * according to the formula
     * repPerBatch = _repRewardConstA * ((_repRewardConstB/1000) ** batchIndex)
     * @param  repRewardConstA
     * @param  repRewardConstB
     * @param  batchIndex the batchIndex to calculate
     * @return     RepuationRewardForBatch
     */
    CL4RScheme.prototype.getRepuationRewardForBatch = function (repRewardConstA, repRewardConstB, batchIndex) {
        return __awaiter(this, void 0, void 0, function () {
            var constB, constA;
            return __generator(this, function (_a) {
                constB = new decimal_js_1.Decimal(utils_1.realMathToNumber(new BN(repRewardConstB)));
                constA = new decimal_js_1.Decimal(utils_1.realMathToBN(new BN(repRewardConstA)).toString());
                return [2 /*return*/, constA.times(constB.toPower(batchIndex))];
            });
        });
    };
    CL4RScheme.prototype.getAgreementHash = function () {
        return __awaiter(this, void 0, void 0, function () {
            var contract, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getContract()];
                    case 1:
                        contract = _a.sent();
                        return [4 /*yield*/, contract.methods.getAgreementHash().call()];
                    case 2:
                        result = _a.sent();
                        return [2 /*return*/, result];
                }
            });
        });
    };
    CL4RScheme.prototype.getReputationRewardForLockingIds = function (lockingIds, batchIndex, repuationRewardForPeriod) {
        return __awaiter(this, void 0, void 0, function () {
            var reputation, contract, lockingTotalScore, _a, _i, lockingIds_1, lockingId, lockingIdScore, _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        reputation = new decimal_js_1.Decimal('0');
                        return [4 /*yield*/, this.getContract()];
                    case 1:
                        contract = _c.sent();
                        _a = decimal_js_1.Decimal.bind;
                        return [4 /*yield*/, contract.methods.batches(batchIndex).call()];
                    case 2:
                        lockingTotalScore = new (_a.apply(decimal_js_1.Decimal, [void 0, _c.sent()]))();
                        if (lockingTotalScore.isZero()) {
                            return [2 /*return*/, lockingTotalScore];
                        }
                        _i = 0, lockingIds_1 = lockingIds;
                        _c.label = 3;
                    case 3:
                        if (!(_i < lockingIds_1.length)) return [3 /*break*/, 6];
                        lockingId = lockingIds_1[_i];
                        _b = decimal_js_1.Decimal.bind;
                        return [4 /*yield*/, contract.methods.getLockingIdScore(batchIndex, lockingId).call()];
                    case 4:
                        lockingIdScore = new (_b.apply(decimal_js_1.Decimal, [void 0, _c.sent()]))();
                        reputation = reputation.add(lockingIdScore.div(lockingTotalScore).mul(repuationRewardForPeriod));
                        _c.label = 5;
                    case 5:
                        _i++;
                        return [3 /*break*/, 3];
                    case 6: return [2 /*return*/, reputation];
                }
            });
        });
    };
    CL4RScheme.prototype.lock = function (amount, period, batchIndexToLockIn, agreementHash) {
        var _this = this;
        var mapReceipt = function (receipt) {
            return receipt;
        };
        var observable = rxjs_1.from(this.getContract())
            .pipe(operators_1.concatMap(function (contract) {
            var transaction;
            transaction = contract.methods.lock(amount, period, batchIndexToLockIn, agreementHash);
            var errorHandler = function (error) { return __awaiter(_this, void 0, void 0, function () {
                var err_1;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            _a.trys.push([0, 2, , 3]);
                            return [4 /*yield*/, transaction.call()];
                        case 1:
                            _a.sent();
                            return [3 /*break*/, 3];
                        case 2:
                            err_1 = _a.sent();
                            throw err_1;
                        case 3: return [2 /*return*/, error];
                    }
                });
            }); };
            return _this.scheme.context.sendTransaction(transaction, mapReceipt, errorHandler);
        }));
        return operation_1.toIOperationObservable(observable);
    };
    CL4RScheme.prototype.extendLocking = function (extendPeriod, batchIndexToLockIn, lockingId, agreementHash) {
        var _this = this;
        var mapReceipt = function (receipt) {
            return receipt;
        };
        var observable = rxjs_1.from(this.getContract())
            .pipe(operators_1.concatMap(function (contract) {
            var transaction;
            transaction = contract.methods.extendLocking(extendPeriod, batchIndexToLockIn, lockingId, agreementHash);
            var errorHandler = function (error) { return __awaiter(_this, void 0, void 0, function () {
                var err_2;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            _a.trys.push([0, 2, , 3]);
                            return [4 /*yield*/, transaction.call()];
                        case 1:
                            _a.sent();
                            return [3 /*break*/, 3];
                        case 2:
                            err_2 = _a.sent();
                            throw err_2;
                        case 3: return [2 /*return*/, error];
                    }
                });
            }); };
            return _this.scheme.context.sendTransaction(transaction, mapReceipt, errorHandler);
        }));
        return operation_1.toIOperationObservable(observable);
    };
    CL4RScheme.prototype.release = function (beneficiary, lockingId) {
        var _this = this;
        var mapReceipt = function (receipt) {
            return receipt;
        };
        var observable = rxjs_1.from(this.getContract())
            .pipe(operators_1.concatMap(function (contract) {
            var transaction;
            transaction = contract.methods.release(beneficiary, lockingId);
            var errorHandler = function (error) { return __awaiter(_this, void 0, void 0, function () {
                var err_3;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            _a.trys.push([0, 2, , 3]);
                            return [4 /*yield*/, transaction.call()];
                        case 1:
                            _a.sent();
                            return [3 /*break*/, 3];
                        case 2:
                            err_3 = _a.sent();
                            throw err_3;
                        case 3: return [2 /*return*/, error];
                    }
                });
            }); };
            return _this.scheme.context.sendTransaction(transaction, mapReceipt, errorHandler);
        }));
        return operation_1.toIOperationObservable(observable);
    };
    /**
     * redeem redeem reputation for a beneficiary for all is lockingIds
     * @param  beneficiary
     * @param  lockingIds
     * @param  network network is a optional.
     * @return     RepuationRewardForBatch
     */
    CL4RScheme.prototype.redeem = function (beneficiary, lockingIds, ct4rRedeemerAddress) {
        var _this = this;
        var mapReceipt = function (receipt) {
            return receipt;
        };
        var ctl4rReedeems = [];
        for (var _i = 0, lockingIds_2 = lockingIds; _i < lockingIds_2.length; _i++) {
            var lockingId = lockingIds_2[_i];
            ctl4rReedeems.push({ beneficiary: beneficiary, id: lockingId.toString() });
        }
        var observable = rxjs_1.from(this.getCT4RRedeemer(ct4rRedeemerAddress ? ct4rRedeemerAddress : CT4RRedeemerAddress))
            .pipe(operators_1.concatMap(function (contract) {
            var transaction;
            transaction = contract.methods.redeemContinuousLocking4Reputation(_this.scheme.staticState.address, ctl4rReedeems);
            var errorHandler = function (error) { return __awaiter(_this, void 0, void 0, function () {
                var err_4;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            _a.trys.push([0, 2, , 3]);
                            return [4 /*yield*/, transaction.call()];
                        case 1:
                            _a.sent();
                            return [3 /*break*/, 3];
                        case 2:
                            err_4 = _a.sent();
                            throw err_4;
                        case 3: return [2 /*return*/, error];
                    }
                });
            }); };
            return _this.scheme.context.sendTransaction(transaction, mapReceipt, errorHandler);
        }));
        return operation_1.toIOperationObservable(observable);
    };
    CL4RScheme.prototype.getContract = function () {
        return __awaiter(this, void 0, void 0, function () {
            var address, contract;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getContractAddress()];
                    case 1:
                        address = _a.sent();
                        return [4 /*yield*/, this.scheme.context.fetchContractInfos({ fetchPolicy: 'network-only' })];
                    case 2:
                        _a.sent();
                        contract = this.scheme.context.getContract(address);
                        return [2 /*return*/, contract];
                }
            });
        });
    };
    CL4RScheme.prototype.getContractAddress = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.scheme.fetchStaticState()];
                    case 1: return [2 /*return*/, (_a.sent()).address];
                }
            });
        });
    };
    CL4RScheme.prototype.getCT4RRedeemer = function (ct4rRedeemerAddress) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, new this.scheme.context.web3.eth.Contract(CT4RRedeemerABI, ct4rRedeemerAddress)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    CL4RScheme.prototype.getScheme = function () {
        return this.scheme;
    };
    return CL4RScheme;
}());
exports.CL4RScheme = CL4RScheme;
//# sourceMappingURL=cl4rep.js.map